#!/usr/bin/env python
###############################################################################
#  Gems.py -- generate gems
###############################################################################

import Dice
import _Treasure

_ornamental_stones = [
        'Azurite',          'Banded Agate',
        'Blue Quartz',      'Eye Agate',
        'Hematite',         'Lapis Lazuli',
        'Malachite',        'Moss Agate',
        'Obsidian',         'Rhodochrosite',
        'Tiger Eye',        'Turquoise',
        'Marble',
]

_semi_precious_stones = [
        'Bloodstone',       'Carnelian',
        'Chalcedony',       'Chrysoprase',
        'Citrine',          'Jasper',
        'Moonstone',        'Onyx',
        'Rock Crystal',     'Sardonyx',
        'Smoky Quartz',     'Star Rose Quartz',
        'Zircon',
]

_fancy_stones = [
        'Amber',            'Alexandrite',
        'Amethyst',         'Chrysoberyl',
        'Coral',            'Garnet',
        'Jade',             'Jet',
        'Peridot',          'Tourmaline',
        'Spinel',           'Pearl',
]

_precious_stones = [
        'Pearl',            'Aquamarine',
        'Spinel',           'Garnet',
        'Topaz',
]

_gems = [
        'Black Opal',       'Fire Opal',
        'Opal',             'Oriental Amethyst',
        'Oriental Topaz',   'Sapphire',
]

_jewels = [
        'Emerald',          'Black Sapphire',
        'Diamond',          'Jacinth',
        'Oriental Emerald', 'Ruby',
        'Star Ruby',        'Star Sapphire',
]

_gem_table = [
    (25, "Ornamental Stones",    _ornamental_stones, 10.0),
    (25, "Semi-Precious Stones", _semi_precious_stones, 50.0),
    (20, "Fancy Stones",         _fancy_stones, 100.0),
    (20, "Precious Stones",      _precious_stones, 500.0),
    (9,  "Gems",                 _gems, 1000.0),
    (1,  "Jewels",               _jewels, 5000.0),
]

del _ornamental_stones
del _semi_precious_stones
del _fancy_stones
del _precious_stones
del _gems
del _jewels

__basevalues = (
    .1, .5, 1.0, 5.0, 
    10.0, 50.0, 100.0, 500.0, 1000.0, 5000.0,
    10000.0, 20000.0, 40000.0, 80000.0, 100000.0,
)

def __adjust_base(base, incr):
    for i in range(len(__basevalues)):
        if __basevalues[i] == base:
            p = i + incr
            if p >= len(__basevalues):
                p = len(__basevalues) - 1
            if p < 0:
                p = 0
            return __basevalues[p]
    raise ValueError, "improper base value %g" % float(base)

def _incr_base(base):
    done = 0
    while not done:
        nbase = __adjust_base(base, 1)
        if nbase == base:
            # short circuit to avoid unneeded extra loops
            return base
        base = nbase
        done = Dice.D(1, 6, 0) - 1
    return base

def _decr_base(base):
    done = 0
    while not done:
        nbase = __adjust_base(base, -1)
        if nbase == base:
            # short circuit to avoid unneeded extra loops
            return base
        base = nbase
        done = Dice.D(1, 6, 0) - 1
    return base

def _dbl_base(base):
    return float(base) * 2

def _half_base(base):
    return float(base) / 2.0

def _imp_base(base):
    return float(base) * \
        (1 + (float(Dice.D(10, 6, 0)) / 100))

def _low_base(base):
    return float(base) * \
        (1 - (float(Dice.D(10, 4, 0)) / 100))

class Gem(_Treasure.Item):
    __cuts = [
        (1, 'Higher Base',  _incr_base),
        (1, 'Double Base',  _dbl_base),
        (1, 'Improve Base', _imp_base),
        (1, 'Reduce Base',  _low_base),
        (1, 'Half Base',    _half_base),
        (1, 'Lower Base',   _decr_base),
    ]
    def __init__(self):
        _Treasure.Item.__init__(self)
        row = Dice.Roll(_gem_table)
        self.name = Dice.Select(row[2])
        self.cat = "Gem"
        self.fullcat = self.fullcat + "." + self.cat
        if Dice.D(1, 100, 0) <= 10:
            cut = Dice.Roll(self.__cuts)
            self.value = cut[2](row[3])
        else:
            self.value = row[3]
        if self.value >= 1:
            self.value = int(self.value) * 1.0
        self.xp = self.value

del _incr_base
del _dbl_base
del _imp_base
del _low_base
del _half_base
del _decr_base

if __name__ == '__main__':
    print Gem()

# end of file.
